from freqtrade.strategy.interface import IStrategy
from pandas import DataFrame
import talib.abstract as ta

class BB_RSI_FVG_Strategy(IStrategy):
    def populate_indicators(self, dataframe: DataFrame, metadata: dict) -> DataFrame:
        # Bollinger Bands %B
        bollinger = ta.BBANDS(dataframe, timeperiod=200)
        dataframe['bb_percent_b'] = (dataframe['close'] - bollinger['lowerband']) / (bollinger['upperband'] - bollinger['lowerband'])
        
        # RSI
        dataframe['rsi'] = ta.RSI(dataframe, timeperiod=50)

        # Fair Value Gap (FVG) approximation
        # Identifying significant jumps in price
        dataframe['fvg'] = dataframe['close'].diff().abs() > (dataframe['close'] * 0.01)  # Example threshold: 1% price jump

        # VWAP for Premium/Discount zones
        dataframe['vwap'] = ta.VWAP(dataframe)

        return dataframe

    def populate_buy_trend(self, dataframe: DataFrame, metadata: dict) -> DataFrame:
        dataframe.loc[
            (
                (dataframe['rsi'] < 30) &
                (dataframe['bb_percent_b'] < 0) &
                (dataframe['fvg']) &  # Buy in FVG zones
                (dataframe['close'] < dataframe['vwap'])  # Buy in Discount zones
            ),
            'buy'] = 1
        return dataframe

    def populate_sell_trend(self, dataframe: DataFrame, metadata: dict) -> DataFrame:
        dataframe.loc[
            (
                (dataframe['rsi'] > 70) &
                (dataframe['bb_percent_b'] > 1) &
                (dataframe['fvg']) &  # Sell in FVG zones
                (dataframe['close'] > dataframe['vwap'])  # Sell in Premium zones
            ),
            'sell'] = 1
        return dataframe
